/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Hultmann Extensions.
 *
 * The Initial Developer of the Original Code is
 * Jeferson Hultmann <hultmann@gmail.com>
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

const EXPORTED_SYMBOLS = ['popupCore'];

Components.utils.import("resource://hultmann-js-migre-me/util.js");


function popupCore(popup, myTarget) {

  var doc = popup.ownerDocument;
  var clickedNode = doc.popupNode;
  var pos = doc.popupRangeOffset;


  if (isInputNode(clickedNode) === false) {

    // Link

    var url;

    if (myTarget.id !== 'contentAreaContextMenu') {
      return;
      /*
      // chrome menu?
      url = getAnchor(clickedNode); // clickedNode may be invalid
      if (url === null) {
        return;
      } */
    } else {
      var contextMenu = doc.defaultView.gContextMenu;
      if (contextMenu.onLink === false) {
        return;
      }
      url = contextMenu.linkURL;
    }

    var uri = toUri(url);
    if (uri === null) {
      return;
    }

    var linkHost = uri.host;
    if (util.isShortHost(linkHost) === false) {
      return;
    }

    if (uri.path.length === 1) { // '/'
      return;
    }

    // migre.me, is.gd, bit.ly, ...
    Components.utils.import("resource://hultmann-js-migre-me/menu.js");
    PUBLIC_menuTinyUrlLink(popup, uri);


  } else {


    // input

    if (isEditMenu(popup) === false) {
      return;
    }

    var addr = extractWord(doc);
    if (addr === null) {
      return;
    }

    var uri = toUri(addr);
    if (uri === null) {
      return;
    }

    var cmdData = {
      uri: uri,
      typed: addr,
      node: clickedNode,
      selectionStart: clickedNode.selectionStart,
      selectionEnd: clickedNode.selectionEnd
    };

    Components.utils.import("resource://hultmann-js-migre-me/menu.js");
    PUBLIC_menuInput(popup, cmdData, myTarget.id === 'urlbar');
  }

  // show commands
  popup.addEventListener('popuphidden', onPopupHidden, false);
}


function onPopupHidden(evt) {
  evt.originalTarget.removeEventListener('popuphidden', onPopupHidden, false);

  var doc = evt.originalTarget.ownerDocument;
  var removeMenuItem = function(id) {
    var el = doc.getElementById(id);
    if (el) {
      el.parentNode.removeChild(el);
    }
  }

  removeMenuItem('hultmann-migre-me-cascade');
  removeMenuItem('hultmann-migre-me-convert');
  removeMenuItem('hultmann-migre-me-separator');
}

/*
function getAnchor(clickedNode) {
  var findAnchor = function(elem) {
    if (elem === null) {
      return null;
    }
    var node = elem;
    while (node.tagName) {
      //util.log('this node '+node.tagName + (typeof node) + node.toString() + node.toSource() + (el instanceof Components.interfaces.nsIDOMXULElement));
      util.log('this node '+node.tagName+node.href);
      //if (node.tagName === 'A') {
      //if (node instanceof Components.interfaces.nsIDOMHTMLAnchorElement && node.href) {
      if (node.href) {
        util.log('found');
        return node.href;
      }
      if (node.hasAttribute('href')) {
        util.log('found');
        return node.getAttribute('href');
      }
      node = node.parentNode;
    }
    return null;
  };
  return findAnchor(clickedNode);
}
*/

function extractWord(doc) {
  var clickedNode = doc.popupNode;
  var txtNode;
  var isSelection = clickedNode.selectionStart !== clickedNode.selectionEnd;
  if (isSelection) {
    txtNode = clickedNode.value.substring(clickedNode.selectionStart, clickedNode.selectionEnd);
    if (txtNode.indexOf('\n') > - 1) {
      return null;
    }
    if (txtNode !== util.trim(txtNode)) {
      return null;
    }
    return txtNode;
  }


  var addr;
  // if empty=NaN; empty input|empty region textatra=node.data=undefined;
  var pos = doc.popupRangeOffset;
  txtNode = isNaN(pos) || doc.popupRangeParent === null ? undefined : doc.popupRangeParent.data;
  if (txtNode === undefined) {
    util.log('no txt,' + doc.popupRangeParent + pos);
    return null;
  }

  var wordIdx = util.findCurrentUrlPos(txtNode, pos);
  return txtNode.substr(wordIdx[0], wordIdx[1]);
}


function isInputNode(clickedNode) {
  if (!clickedNode) {
    return false;
  }
  //document.designMode === 'on'
  switch (clickedNode.tagName) {
    case 'INPUT':
    case 'html:input':
      return clickedNode.type === 'text';
    case 'TEXTAREA':
    case 'html:textarea':
      return true;
    default:
      return false;
  }
}


function isEditMenu(menu) {
  var items = menu.getElementsByTagNameNS('http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul', 'menuitem');
  var ok = false;
  for (var idx = 0; idx < items.length; idx++) {
    var item = items[idx];

    var id = '';
    if (item.hasAttribute('command')) {
      id = item.getAttribute('command');
    } else if (item.hasAttribute('cmd')) {
      id = item.getAttribute('cmd');
    } else {
      continue;
    }

    if (id === 'cmd_find') {
      return false;
    }
    if (id === 'cmd_cut') {
      ok = true;
    }
  }
  return ok;
}

function toUri(txt) {
  var uri = null;
  if (txt.indexOf('www.') === 0) {
    uri = util.stringToUri('http://' + txt);
  } else {
    uri = util.stringToUri(txt);
  }

  if (uri === null) {
    return null;
  }

  switch (uri.scheme) {
    case 'http':
    case 'https':
    case 'ftp':
    case 'gopher':
      return uri;
  }
  return null;
}
