/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Hultmann Extensions.
 *
 * The Initial Developer of the Original Code is
 * Jeferson Hultmann <hultmann@gmail.com>
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

const EXPORTED_SYMBOLS = ['util'];

const util = {
  isShortHost: function(host) {
    switch (host) {
      case 'migre.me': // <== api-migreme.js

      case 'bit.ly':
      case 'tinyurl.com':
      case 'cli.gs':
      case 'digg.com':
      case 'is.gd':
      case 'moourl.com':
      case 'twurl.nl':
      case 'ff.im':
      case '307.to':
      case 'adjix.com':
      case 'b23.ru':
      case 'bacn.me':
      case 'bloat.me':
      case 'budurl.com':
      case 'clipurl.us':
      case 'cort.as':
      case 'dwarfurl.com':
      case 'kl.am':
      case 'fff.to':
      case 'href.in':
      case 'idek.net':
      case 'korta.nu':
      case 'lin.cr':
      case 'ln-s.net':
      case 'tr.im':
      case 'loopt.us':
      case 'memurl.com':
      case 'merky.de':
      case 'nanourl.se':
      case 'ow.ly':
      case 'peaurl.com':
      case 'ping.fm':
      case 'piurl.com':
      case 'plurl.me':
      case 'pnt.me':
      case 'poprl.com':
      case 'rde.me':
      case 'reallytinyurl.com':
      case 'redir.ec':
      case 'rubyurl.com':
      case 'short.ie':
      case 'short.to':
      case 'smallr.com':
      case 'sn.vc':
      case 'snipr.com':
      case 'snipurl.com':
      case 'snurl.com':
      case 'tiny.cc':
      case 'togoto.us':
      case 'tra.kz':
      case 'twurl.cc':
      case 'u.mavrev.com':
      case 'u.nu':
      case 'ur1.ca':
      case 'url.az':
      case 'url.ie':
      case 'urlx.ie':
      case 'w34.us':
      case 'xrl.us':
      case 'yep.it':
      case 'zi.ma':
      case 'zurl.ws':
      case 'chilp.it':
      case 'notlong.com':
      case 'qlnk.net':
        return true;
      default:
        return false;
    }
  },

  _toArray: function(webOnly, theStack) {
    var allItems = [];
    if (!theStack) {
      //$.moz.prompt('theStack undef\n'+$.stack.toString(false, Components.stack))
      return allItems;
    }

    function StackItem() {
    }
    StackItem.prototype = {
      lang: '?',
      filename: '?',
      lineNumber: -1,
      name: '?'
    };

    // aparentemente é um bug que ocorre as vezes, todas as propriedades sao undefined
    if (theStack.languageName == undefined) {
      //$.moz.prompt('myStack\n'+myStack +  '\n\ntheStack\n'+theStack + '\n\nallItems\n' + allItems.join('\n') + '\n\n\n'+$.stack.toString(false, Components.stack))
      var item = new StackItem();
      item.name = theStack.toString();
      allItems.push(item);
    }

    //for (var myStack = theStack; myStack !== null; myStack = myStack.caller) {
    //    $.moz.prompt('myStack\n'+myStack);
    //}

    // myStack.caller é quase sempre null, mas algumas vezes é undefined...
    for (var myStack = theStack; myStack; myStack = myStack.caller) {
      if (webOnly) {
        var n = myStack.filename;
        if (n === null) {
          continue;
        }
        if ((n.indexOf('http:') !== 0) && (n.indexOf('https:') !== 0)) {
          continue;
        }
      }


      var item = new StackItem();
      allItems.push(item);
      item.lang = myStack.languageName;
      item.filename = myStack.filename;
      item.lineNumber = myStack.lineNumber; // Valid line numbers begin at '1'. '0' indicates unknown.
      item.name = myStack.name;
    }

    allItems.reverse();
    return allItems;
  },

  _toString: function(webOnly, theStack) {
    var arr = this._toArray(webOnly, theStack);
    var lines = new Array(arr.length);
    for (var idx = 0, len = arr.length; idx < len; idx++) {
      var lang = webOnly ? '' : arr[idx].lang + ' ';
      lines[idx] = '[' + (idx + 1) + '] '
                  + lang
                  + arr[idx].filename + ' '
                  + arr[idx].name + ' ('
                  + arr[idx].lineNumber + ')';
    }
    return lines.join('\n');
  },

  myThrow: function(txt) {
    this.log('Stack:\n' + this._toString(false, Components.stack));
    throw txt;
  },


  trim: function(txt) {
    txt = txt.replace(/^\s+/, '');
    for (var idx = txt.length - 1; idx > 0; idx--) {
      if (/\S/.test(txt.charAt(idx))) {
        txt = txt.substring(0, idx + 1);
        break;
      }
    }
    return txt;
  },

  stringToUri: function(spec) {
    try {
      return Components
              .classes['@mozilla.org/network/io-service;1']
              .getService(Components.interfaces.nsIIOService)
              .newURI(spec, null, null);
    } catch (ex) {
      return null;
    }
  },

  getText: function(msg) {
    var qty = arguments.length - 1;
    var args = new Array(qty);
    for (var idx = 0; idx < qty; idx++) {
      args[idx] = arguments[idx + 1];
    }
    return Components
            .classes['@mozilla.org/intl/stringbundle;1']
            .getService(Components.interfaces.nsIStringBundleService)
            .createBundle('chrome://migre-me-hultmann/locale/general.properties')
            .formatStringFromName(msg, args, args.length);
  },

  log: function(msg) {
    var now = new Date();
    var p = '[migre.me' + ' ' + now.toLocaleTimeString() + '] ';
    dump('\n------------------------------------------------------------------\n' + p + msg);
    Components
      .classes['@mozilla.org/consoleservice;1']
      .getService(Components.interfaces.nsIConsoleService)
      .logStringMessage(p + msg);
  },

  findCurrentUrlPos: function(txt, idx) {
    //var bounds = ['http:', 'https:', 'ftp:', 'www.'];
    var bounds = [' ', '\n', '\t', '(', ')', '"', "'", "<", ">", '[', ']'];
    var mySort = function(a, b) {
      return a - b;
    };

    var stopChars = bounds.map(function(myChar) {
      var idx2 = txt.indexOf(myChar, idx);
      return idx2 === -1 ? Infinity : idx2;
    });


    stopChars.sort(mySort);
    var endNearest = stopChars[0];
    var lastChar = (endNearest === Infinity ? txt.length : endNearest) - 1;


    var startChars = bounds.map(function(myChar) {
      return txt.lastIndexOf(myChar, lastChar);
    });

    startChars.sort(mySort);
    var startNearest = startChars[startChars.length - 1];

    var firstChar = startNearest == -1 ? 0 : startNearest + 1;
    return [firstChar, lastChar - firstChar + 1];
  }

};
